{***************************************************************
 *
 * Unit Name : CharsetEnumerator
 * Purpose   : A class that lists all CHARSETs for a font or
               list all fonts that support a specific CHARSET
 * Author    : Ralf Kaiser
 * Copyright :  2001-2009 by Ralf Kaiser
 * History   : -
 * Comment   : -
 *

 Licence :

 Copyright 2001-2009, ralf Kaiser.  All Rights Reserved.

 Copyright:
 This unit source code, as part ot the "EmperorTools", is copyrighted by Ralf Kaiser
 (hereafter "author"), and shall remain the exclusive property of the author.

 Distribution Rights:
 You are granted a non-exlusive, royalty-free right to produce and distribute
 compiled binary files (executables, DLLs, etc.) that are built with EmperorTools.

 You are further granted permission to redistribute any of the EmperorTools source
 code in source code form, provided that the original archive as found on the
 web site (http://www.emperortools.de) is distributed unmodified. For example,
 if you create a descendant of "TCharsetEnumerator", you must include in
 the distribution package the "CharsetEnum.zip" file in the exact form that you
 downloaded it from http://www.emperortools.de.

 Restrictions:
 Without the express written consent of the author, you may not:

   * Distribute modified versions of any "EmperorTools" source code by itself. You must
     include the original archive as you found it at the website.
   * Sell or lease any portion of "EmperorTools" source code. You are, of course, free
     to sell any of your own original code that works with, enhances, etc.
     "EmperorTools" source code.
   * Distribute "EmperorTools" source code for profit.

 Warranty:
 There is absolutely no warranty of any kind whatsoever with any of the DFS
 source code (hereafter "software"). The software is provided to you "AS-IS",
 and all risks and losses associated with it's use are assumed by you. In no
 event shall the author of the softare, Ralf Kaiser, be held accountable for
 any damages or losses that may occur from use or misuse of the software.

 Support:
 Currently the author does not provide any special support for the "EmperorTools".
 For new versions check the web site "http://www.emperortools.de".

 Clarifications:
 If you need any further information, please feel free to contact me directly. See
 "http://www.emperortools.de" for infos regarding e-mail-address details.

****************************************************************}

unit CharsetEnumerator;

interface

uses
  SysUtils,
  Classes,
  Graphics,
  Windows;

type

  TCharsetEnumerator = class(TObject)
  private
    FCharSetNames: TStrings;
    FCharSetNummern: TStrings;
    FFontCanvasHandle: HDC;
    FFontCharset: TFontCharset;
    FFontName: TFontName;
    FFontNameList: TStrings;
    FLogFont: TLogFont;
    FWesternCharSetName: string;
    procedure ClearLists;
    function GetFontCharsetNames: TStrings;
    function GetFontNames: TStrings;
    procedure Refresh;
    procedure SetFontCanvasHandle(Value: HDC);
    procedure SetFontCharset(Value: TFontCharset);
    procedure SetFontName(Value: TFontName);
  public
    constructor Create;
    destructor Destroy; override;
    property FontCanvasHandle: HDC read FFontCanvasHandle write SetFontCanvasHandle;
    property FontCharset: TFontCharset write SetFontCharset;
    property FontCharsetNames: TStrings read GetFontCharsetNames;
    property FontName: TFontName write SetFontName;
    property FontNames: TStrings read GetFontNames;
    property LogFont: TLogFont read FLogFont write FLogFont;
  end;

implementation

constructor TCharsetEnumerator.Create;
{******************* TCharsetEnumerator.Create *************************

Method       : TCharsetEnumerator.Create
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : None
Returnvalues : None
Version      : 1.0
Purpose      : initializes the internal lists
Changes      :

******************* TCharsetEnumerator.Create *************************}
begin
  FFontNameList := TStringList.Create;
  FCharSetNames := TStringList.Create;
  FCharSetNummern := TStringList.Create;
  FontCanvasHandle := 0;                // initialize the canvas handle with the handle of the desktop canvas
end;

destructor TCharsetEnumerator.Destroy;
{******************* TCharsetEnumerator.Destroy *************************

Method       : TCharsetEnumerator.Destroy
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : None
Returnvalues : None
Version      : 1.0
Purpose      : frees the memory of the internal lists
Changes      :

******************* TCharsetEnumerator.Destroy *************************}
begin
  if FFontNameList <> nil then begin    // ferr the internal lists
    FreeAndNil(FFontNameList);
  end;
  if FCharSetNames <> nil then begin
    FreeAndNil(FCharSetNames);
  end;
  if FCharSetNummern <> nil then begin
    FreeAndNil(FCharSetNummern);
  end;
  inherited Destroy;                    // call the original destructor of TObject ...
end;

procedure TCharsetEnumerator.ClearLists;
{******************* TCharsetEnumerator.ClearLists *************************

Method       : TCharsetEnumerator.ClearLists
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : None
Returnvalues : None
Version      : 1.0
Purpose      : Clears the internal lists
Changes      :

******************* TCharsetEnumerator.ClearLists *************************}
begin
  FFontNameList.Clear;
  FCharSetNames.Clear;
  FCharSetNummern.Clear;
end;

function TCharsetEnumerator.GetFontCharsetNames: TStrings;
{******************* TCharsetEnumerator.GetFontCharsetNames *************************

Method       : TCharsetEnumerator.GetFontCharsetNames
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : None
Returnvalues : TStrings
Version      : 1.0
Purpose      : Returns the names of the charactersets for a specific font
               (or for all fonts that were listed)
Changes      :

******************* TCharsetEnumerator.GetFontCharsetNames *************************}
var
  i                                     : Integer;
begin
  Refresh;
  Result := FCharSetNames;
end;

function TCharsetEnumerator.GetFontNames: TStrings;
{******************* TCharsetEnumerator.GetFontNames *************************

Method       : TCharsetEnumerator.GetFontNames
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : None
Returnvalues : TStrings
Version      : 1.0
Purpose      : Returns all font names that support a specific character set
               (or all character sets for all fonts)
Changes      :

******************* TCharsetEnumerator.GetFontNames *************************}
begin
  Refresh;
  Result := FFontNameList;
end;

procedure TCharsetEnumerator.Refresh;
{******************* TCharsetEnumerator.Refresh *************************

Method       : TCharsetEnumerator.Refresh
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : None
Returnvalues : None
Version      : 1.0
Purpose      : Reads the font and character set informations and
               stores them in the internal lists
Changes      :

******************* TCharsetEnumerator.Refresh *************************}
  function ProcessOneFont(EnumLogFont: PEnumLogFontEx; TextMetrics: PNEWTEXTMETRIC; FontType: Integer; var EnumObject: TCharsetEnumerator): longint; stdcall;
  begin
    if (EnumObject.FFontName <> '') and (EnumObject.FFontCharset = DEFAULT_CHARSET) then begin
      if EnumObject.FFontName = EnumLogFont.elfFullName then begin

        if EnumLogFont.elfLogFont.lfCharSet = 0 then
          EnumObject.FWesternCharSetName := EnumLogFont.elfScript;
        EnumObject.FCharSetNummern.Add(inttostr(EnumLogFont.elfLogFont.lfCharSet));

        EnumObject.FCharSetNames.Add(EnumLogFont.elfScript);
      end;
    end else if (EnumObject.FFontName = '') and (EnumObject.FFontCharset <> DEFAULT_CHARSET) then begin
      EnumObject.FFontNameList.Add(EnumLogFont.elfFullName);
    end else begin
      EnumObject.FCharSetNames.Add(EnumLogFont.elfScript);
      EnumObject.FFontNameList.Add(EnumLogFont.elfFullName);
    end;
    Result := -1;
  end;
begin
  // copy the font name we want to examine to the name field of the structure
  StrPCopy(FLogFont.lfFaceName, FFontName);
  // set the character set field of the structure
  FLogFont.lfCharSet := FFontCharset;
  FCharSetNames.Clear;
  // read all specified info with the callback function "ProcessOneFont"
  // a reference to the current class instance is passed through the
  // application-specific LPARAM-Value (parameter #4)
  EnumFontFamiliesEx(FFontCanvasHandle, FLogFont, @ProcessOneFont, Integer(@Self), 0);
  if FCharSetNames.Count < 1 then begin
    FCharSetNames.Add(FWesternCharSetName);
    FCharSetNummern.Add('0');
  end;
end;

procedure TCharsetEnumerator.SetFontCanvasHandle(Value: HDC);
{******************* TCharsetEnumerator.SetFontCanvasHandle *************************

Method       : TCharsetEnumerator.SetFontCanvasHandle
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : Value: HDC
Returnvalues : None
Version      : 1.0
Purpose      : Stores the handle of the device context to be examined,
               if the value is zero, it reads the device context handle
               of the desktop canvas.
Changes      : change this property to read the info for other devices
               such as printers

******************* TCharsetEnumerator.SetFontCanvasHandle *************************}
begin
  if Value = 0 then begin
    FFontCanvasHandle := GetDC(0);
  end else begin
    FFontCanvasHandle := Value;
  end;
end;

procedure TCharsetEnumerator.SetFontCharset(Value: TFontCharset);
{******************* TCharsetEnumerator.SetFontCharset *************************

Method       : TCharsetEnumerator.SetFontCharset
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : Value: TFontCharset
Returnvalues : None
Version      : 1.0
Purpose      : sets the value for the characterset that the class looks for
               and clears the font name
Changes      :

******************* TCharsetEnumerator.SetFontCharset *************************}
begin
  FFontName := '';
  FFontCharset := Value;
  ClearLists;
end;

procedure TCharsetEnumerator.SetFontName(Value: TFontName);
{******************* TCharsetEnumerator.SetFontName *************************

Method       : TCharsetEnumerator.SetFontName
Author       : Ralf Kaiser
Date         : 15.04.2001
Arguments    : Value: TFontName
Returnvalues : None
Version      : 1.0
Purpose      : sets the value for the font name that the class looks for
               and clears the character set
Changes      :

******************* TCharsetEnumerator.SetFontName *************************}
begin
  FFontName := Value;
  FFontCharset := DEFAULT_CHARSET;
  ClearLists;
end;

end.

